/* Integrating DJ Native Swing into NetBeans RCP
 *
 * See the DJ Project by Christopher Deckers
 * http://djproject.sourceforge.net/ns/index.html
 *
 * Geertjaan Wielenga "Step-by-Step Instructions for Integrating DJ Native
 * Swing into NetBeans RCP"
 * http://netbeans.dzone.com/how-to-nb-djnative-swing?utm_source=feedburner&utm_
 * medium=feed&utm_campaign=Feed%3A+zones%2Fnetbeans+%28NetBeans+Zone%29
 *
 * Eclipse swt library
 * http://download.eclipse.org/eclipse/downloads/drops/R-3.6-201006080911/
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence v2.1.
 */
package org.starbeans.modules.nativeswing;

import chrriis.dj.nativeswing.swtimpl.components.JWebBrowser;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserCommandEvent;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserEvent;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserListener;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserNavigationEvent;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserWindowOpeningEvent;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserWindowWillOpenEvent;
import java.awt.Component;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.net.URL;
import javax.swing.SwingUtilities;
import org.openide.awt.HtmlBrowser;
import org.openide.util.Mutex;

/**
 *
 * @author George Waldon
 */
public class DJNSBrowserImpl extends HtmlBrowser.Impl {

    private JWebBrowser webBrowser;
    
    private BrowserListener blistener;
    
    protected PropertyChangeSupport pcs;
    
    public DJNSBrowserImpl() {
        webBrowser = DJNSFactory.createJWebBrowser();
        webBrowser.setBarsVisible(false);
        pcs = new PropertyChangeSupport (this);
        blistener = new BrowserListener();
    }
    
    @Override
    public Component getComponent() {
        return webBrowser;
    }

    @Override
    public void reloadDocument() {
        webBrowser.reloadPage();
    }

    @Override
    public void stopLoading() {
        webBrowser.stopLoading();
    }

    @Override
    public void setURL(final URL url) {
        if (url == null) {
            return;
        }
        
        Mutex.EVENT.readAccess(new Runnable() {
            @Override
            public void run() {
                webBrowser.navigate(url.toString());
            }
        });
        
    }

    @Override
    public URL getURL() {
        if (!SwingUtilities.isEventDispatchThread()) {
            return null;
        }
        String loc = webBrowser.getResourceLocation();
        return IUtil.getURLFromString(loc);
    }

    @Override
    public String getStatusMessage() {
        return webBrowser.getStatusText();
    }

    @Override
    public String getTitle() {
        return webBrowser.getPageTitle();
    }

    @Override
    public boolean isForward() {
        return webBrowser.isForwardNavigationEnabled();
    }

    @Override
    public void forward() {
        webBrowser.navigateForward();
    }

    @Override
    public boolean isBackward() {
        return webBrowser.isBackNavigationEnabled();
    }

    @Override
    public void backward() {
        webBrowser.navigateBack();
    }

    @Override
    public boolean isHistory() {
        return false;
    }

    @Override
    public void showHistory() {
    }

    @Override
    public void addPropertyChangeListener(PropertyChangeListener l) {
        pcs.addPropertyChangeListener (l);
        webBrowser.addWebBrowserListener(blistener);
    }

    @Override
    public void removePropertyChangeListener(PropertyChangeListener l) {
        pcs.removePropertyChangeListener (l);
        webBrowser.removeWebBrowserListener(blistener);
    }
    
    @Override
    public void dispose() {
        webBrowser.disposeNativePeer();
        webBrowser = null;
    }
    
    private class BrowserListener implements WebBrowserListener {

        public BrowserListener() {
        }

        @Override
        public void windowWillOpen(WebBrowserWindowWillOpenEvent wbwwoe) {
        }

        @Override
        public void windowOpening(WebBrowserWindowOpeningEvent wbwoe) {
        }

        @Override
        public void windowClosing(WebBrowserEvent wbe) {
        }

        @Override
        public void locationChanging(WebBrowserNavigationEvent wbne) {
        }

        @Override
        public void locationChanged(WebBrowserNavigationEvent wbne) {
            String newloc = wbne.getNewResourceLocation();
            pcs.firePropertyChange(HtmlBrowser.Impl.PROP_URL, "", newloc);
        }

        @Override
        public void locationChangeCanceled(WebBrowserNavigationEvent wbne) {
        }

        @Override
        public void loadingProgressChanged(WebBrowserEvent wbe) {
        }

        @Override
        public void titleChanged(WebBrowserEvent wbe) {
            JWebBrowser jb = wbe.getWebBrowser();
            String title = jb.getPageTitle();
            if(title!=null && !title.isEmpty()) {
                pcs.firePropertyChange(HtmlBrowser.Impl.PROP_TITLE, "", title);
                pcs.firePropertyChange(HtmlBrowser.Impl.PROP_BACKWARD, "", title);
                pcs.firePropertyChange(HtmlBrowser.Impl.PROP_FORWARD, "", title);
            }
        }

        @Override
        public void statusChanged(WebBrowserEvent wbe) {
        }

        @Override
        public void commandReceived(WebBrowserCommandEvent wbce) {
        }
    }
}
