/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 * Portions Copyrighted 2008-2012 George Waldon
 */
package org.starbeans.modules.corelite.explorer;

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.awt.DynamicMenuContent;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.actions.Presenter;

/**
 *
 * @author George Waldon
 */
@ActionID(id = "org.starbeans.modules.corelite.explorer.OpenDirAction", category = "System") // NOI18N
@ActionRegistration(displayName = "#ACT_opendir", lazy=false)
@ActionReferences(value = {
    @ActionReference(path = "Menu/File", position = 300), // NOI18N
    @ActionReference(path = "Toolbars/File", position = 300)}) // NOI18N
public class OpenDirAction extends AbstractAction implements Presenter.Popup, Presenter.Menu {
    
    private static final Logger LOG = Logger.getLogger(Actions.class.getName());
    
    public OpenDirAction() {
        // Still needed for proper display in toolbar and menu!
        putValue(NAME, NbBundle.getMessage(OpenDirAction.class, "ACT_opendir"));
        Image image = ImageUtilities.loadImage("org/starbeans/modules/corelite/resources/openDir.gif"); // NOI18N
        putValue(SMALL_ICON, new ImageIcon(image));
        putValue("iconBase", "org/starbeans/modules/corelite/resources/openDir.gif"); //NOI18N
    }
        
    @Override
    public void actionPerformed(ActionEvent evt) {

        try {
            // show JFileChooser
            FileObject fo = Actions.Add.chooseFileObject();
            if (fo == null) {
                return;
            }
            final List<DataObject> toShadows = Collections.singletonList(DataObject.find(fo));
            Tab.RP.post(new Runnable() {

                @Override
                public void run() {
                    Actions.Add.addToFavorites(toShadows);
                }
            });

        } catch (DataObjectNotFoundException e) {
            LOG.log(Level.WARNING, null, e);
        }
    }

    @Override
    public JMenuItem getMenuPresenter() {
        return new DynamicMenuItem(this, false);
    }

    @Override
    public JMenuItem getPopupPresenter() {
        return new DynamicMenuItem(this, true);
    }

    private class DynamicMenuItem extends JMenuItem implements DynamicMenuContent {

        private AbstractAction action;
        private boolean popup;

        public DynamicMenuItem(AbstractAction action, boolean popup) {
            this.action = action;
            this.popup = popup;
            org.openide.awt.Actions.connect(this, action, popup);
        }

        @Override
        public JComponent[] getMenuPresenters() {
            JMenuItem menuPresenter = new JMenuItem();
            org.openide.awt.Actions.connect(menuPresenter, action, popup);
            return new JComponent[]{menuPresenter};
        }

        @Override
        public JComponent[] synchMenuPresenters(JComponent[] items) {
            return getMenuPresenters();
        }
    }
}
